﻿
//===========================================================
// Подробное описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//===========================================================

namespace DataGridViewColorCells.Classes
{

    internal class ColorSet
    {
        private readonly List<DataGridView> _dgvs;
        private readonly List<GroupBox> _gbxs;
        private readonly Form _form;
        private readonly MenuStrip _menustrip;

        // Группа сохранения заказных цветов по умолчанию.
        private static readonly List<DataGridViewCellStyle> _defStyles = new();
        private static readonly List<DataGridViewCellStyle> _defAltStyles = new();
        private static readonly List<DataGridViewCellStyle> _defColumnStyles = new();
        private static readonly List<DataGridViewCellStyle> _defRowStyles = new();
        private static readonly List<Color> _defDvgBackgroundColors = new();
        private static bool _init = false;

        public ColorSet(List<DataGridView> dgvs, List<GroupBox> gbxs, Form form, MenuStrip menustrip)
        {
            _dgvs = dgvs;
            _gbxs = gbxs;
            _form = form;
            _menustrip = menustrip;

            // Однократная инициализация статических членов.
            // Сохранение настроек заказных цветов в оперативной памяти.
            if (_init == false)
            {
                for (int i = 0; i < _dgvs.Count; i++)
                {
                    _defDvgBackgroundColors.Add(new Color());
                    _defDvgBackgroundColors[i] = _dgvs[i].BackColor;

                    _defStyles.Add(new DataGridViewCellStyle());
                    _defStyles[i].BackColor = _dgvs[i].DefaultCellStyle.BackColor;
                    _defStyles[i].ForeColor = _dgvs[i].DefaultCellStyle.ForeColor;

                    _defAltStyles.Add(new DataGridViewCellStyle());
                    _defAltStyles[i].BackColor = _dgvs[i].AlternatingRowsDefaultCellStyle.BackColor;
                    _defAltStyles[i].ForeColor = _dgvs[i].AlternatingRowsDefaultCellStyle.ForeColor;

                    _defColumnStyles.Add(new DataGridViewCellStyle());
                    _defColumnStyles[i].BackColor = _dgvs[i].ColumnHeadersDefaultCellStyle.BackColor;
                    _defColumnStyles[i].ForeColor = _dgvs[i].ColumnHeadersDefaultCellStyle.ForeColor;

                    _defRowStyles.Add(new DataGridViewCellStyle());
                    _defRowStyles[i].BackColor = _dgvs[i].ColumnHeadersDefaultCellStyle.BackColor;
                    _defRowStyles[i].ForeColor = _dgvs[i].ColumnHeadersDefaultCellStyle.ForeColor;
                }

                _init = true;
            }
        }

        #region Открытый интерфейс класса

        public void GreenStyle()
        {
            // Сброс цветов к по умолчанию.
            ResetColors();
            ResetFormColor();

            // === Настройка нового стиля ===

            _menustrip.BackColor = Color.FromArgb(152, 230, 152);
            _menustrip.ForeColor = Color.Black;

            _form.BackColor = Color.FromArgb(30, 123, 30);
            _form.ForeColor = Color.White;

            foreach (GroupBox grp in _gbxs)
            {
                grp.ForeColor = Color.White;
            }

            foreach (DataGridView dgv in _dgvs)
            {
                dgv.DefaultCellStyle.SelectionBackColor = Color.FromArgb(0, 128, 255);
                dgv.DefaultCellStyle.SelectionForeColor = Color.White;

                dgv.DefaultCellStyle.BackColor = Color.White;
                dgv.DefaultCellStyle.ForeColor = Color.Black;

                dgv.AlternatingRowsDefaultCellStyle.BackColor = Color.FromArgb(174, 239, 174);
                dgv.AlternatingRowsDefaultCellStyle.ForeColor = Color.Black;

                dgv.ColumnHeadersDefaultCellStyle.BackColor =
                    dgv.RowHeadersDefaultCellStyle.BackColor = Color.FromArgb(20, 82, 20);
                dgv.ColumnHeadersDefaultCellStyle.ForeColor =
                    dgv.RowHeadersDefaultCellStyle.ForeColor = Color.White;

                dgv.BackgroundColor = Color.FromArgb(10, 41, 10);
                dgv.ClearSelection();

                
            }
        }

        public void BlueStyle()
        {
            ResetColors();
            ResetFormColor();

            _menustrip.BackColor = Color.FromArgb(128, 179, 255);
            _menustrip.ForeColor = Color.Black;

            _form.BackColor = Color.FromArgb(0, 71, 179);
            _form.ForeColor = Color.White;


            foreach (GroupBox grp in _gbxs)
            {
                grp.ForeColor = Color.White;
            }

            foreach (DataGridView dgv in _dgvs)
            {
                dgv.DefaultCellStyle.SelectionBackColor = Color.FromArgb(179, 89, 0);
                dgv.DefaultCellStyle.SelectionForeColor = Color.White;

                dgv.DefaultCellStyle.BackColor = Color.White;
                dgv.DefaultCellStyle.ForeColor = Color.Black;

                dgv.AlternatingRowsDefaultCellStyle.BackColor = Color.FromArgb(166, 227, 255);
                dgv.AlternatingRowsDefaultCellStyle.ForeColor = Color.Black;

                dgv.ColumnHeadersDefaultCellStyle.BackColor =
                    dgv.RowHeadersDefaultCellStyle.BackColor = Color.FromArgb(0, 92, 230);
                dgv.ColumnHeadersDefaultCellStyle.ForeColor =
                    dgv.RowHeadersDefaultCellStyle.ForeColor = Color.White;

                dgv.BackgroundColor = Color.FromArgb(0, 51, 128);
                dgv.ClearSelection();
            }
        }

        public void OrangeStyle()
        {
            ResetColors();
            ResetFormColor();

            _menustrip.BackColor = Color.FromArgb(255, 163, 102);
            _menustrip.ForeColor = Color.Black;


            _form.BackColor = Color.FromArgb(128, 51, 0);
            _form.ForeColor = Color.White;

            foreach (DataGridView dgv in _dgvs)
            {
                dgv.DefaultCellStyle.SelectionBackColor = Color.FromArgb(153, 102, 51);
                dgv.DefaultCellStyle.ForeColor = Color.White;

                dgv.DefaultCellStyle.BackColor = Color.White;
                dgv.DefaultCellStyle.ForeColor = Color.Black;

                dgv.AlternatingRowsDefaultCellStyle.BackColor = Color.FromArgb(255, 209, 179);
                dgv.AlternatingRowsDefaultCellStyle.ForeColor = Color.Black;

                dgv.ColumnHeadersDefaultCellStyle.BackColor = 
                    dgv.RowHeadersDefaultCellStyle.BackColor = Color.FromArgb(204, 82, 0);
                dgv.ColumnHeadersDefaultCellStyle.ForeColor =
                    dgv.RowHeadersDefaultCellStyle.ForeColor = Color.White;

                dgv.BackgroundColor = Color.FromArgb(77, 31, 0);
                dgv.ClearSelection();
            }
        }

        public void GrayStyle()
        {
            ResetColors();
            ResetFormColor();

            // ===
            _menustrip.BackColor = Color.FromArgb(190, 190, 190);
            _menustrip.ForeColor = Color.Black;

            _form.BackColor = Color.FromArgb(110, 110, 110);
            _form.ForeColor = Color.White;

            foreach (GroupBox grp in _gbxs)
            {
                grp.ForeColor = Color.White;
            }

            foreach (DataGridView dgv in _dgvs)
            {
                dgv.DefaultCellStyle.SelectionBackColor = Color.FromArgb(0, 153, 153);
                dgv.DefaultCellStyle.ForeColor = Color.White;

                dgv.DefaultCellStyle.BackColor = Color.White;
                dgv.DefaultCellStyle.ForeColor = Color.Black;

                dgv.AlternatingRowsDefaultCellStyle.BackColor = Color.FromArgb(224, 224, 224);
                dgv.AlternatingRowsDefaultCellStyle.ForeColor = Color.Black;

                dgv.ColumnHeadersDefaultCellStyle.BackColor =
                    dgv.RowHeadersDefaultCellStyle.BackColor = Color.FromArgb(86, 86, 86);
                dgv.ColumnHeadersDefaultCellStyle.ForeColor =
                    dgv.RowHeadersDefaultCellStyle.ForeColor = Color.White;

                dgv.BackgroundColor = Color.FromArgb(48, 48, 48);
                dgv.ClearSelection();
            }
        }

        public void DarkStyle()
        {
            ResetColors();
            ResetFormColor();

            _menustrip.BackColor = Color.FromArgb(30, 30, 30);
            _menustrip.ForeColor = Color.White;

            _form.BackColor = Color.FromArgb(90, 90, 90);
            _form.ForeColor = Color.White;

            foreach (GroupBox grp in _gbxs)
            {
                grp.ForeColor = Color.White;
            }

            foreach (DataGridView dgv in _dgvs)
            {
                dgv.DefaultCellStyle.SelectionBackColor = Color.FromArgb(0, 0, 255);
                dgv.DefaultCellStyle.SelectionForeColor = Color.White;

                dgv.DefaultCellStyle.BackColor = Color.FromArgb(80, 80, 80);
                dgv.DefaultCellStyle.ForeColor = Color.White;


                dgv.AlternatingRowsDefaultCellStyle.BackColor = Color.FromArgb(50, 50, 50);
                dgv.AlternatingRowsDefaultCellStyle.ForeColor = Color.White;

                dgv.ColumnHeadersDefaultCellStyle.BackColor =
                    dgv.RowHeadersDefaultCellStyle.BackColor = Color.FromArgb(30, 30, 30);
                dgv.ColumnHeadersDefaultCellStyle.ForeColor =
                    dgv.RowHeadersDefaultCellStyle.ForeColor = Color.White;


                dgv.BackgroundColor = Color.FromArgb(0, 0, 0);
                dgv.ClearSelection();
            }
        }

        public void ColumnStyle()
        {
            ResetColors();

            Color[] colors = { Color.LightCoral, Color.LightGreen, Color.LightBlue, Color.Yellow, Color.Orange, Color.Aqua };
            foreach (DataGridView dgv in _dgvs)
            {
                int count = 0;
                foreach (DataGridViewColumn column in dgv.Columns)
                {
                    column.DefaultCellStyle.BackColor = colors[count];

                    count++;
                }

                dgv.ClearSelection();
            }
        }

        public void RowStyle()
        {
            ResetColors();

            Color[] colors = { Color.LightCoral, Color.LightGreen, Color.LightBlue, Color.Yellow, Color.Orange, Color.Aqua };
            foreach (DataGridView dgv in _dgvs)
            {
                int count = 0;
                foreach (DataGridViewRow row in dgv.Rows)
                {
                    row.DefaultCellStyle.BackColor = colors[count];
                    _ = count < 5 ? count++ : count = 0;
                }

                dgv.ClearSelection();
            }
        }

        public void CellStyle()
        {
            ResetColors();

            Color[] colors = { Color.LightCoral, Color.LightGreen, Color.LightBlue, Color.Yellow, Color.Orange, Color.Aqua };
            foreach (DataGridView dgv in _dgvs)
            {
                foreach (DataGridViewRow row in dgv.Rows)
                {
                    int count = 0;
                    foreach (DataGridViewCell cell in row.Cells)
                    {
                        cell.Style.BackColor = colors[count];
                        _ = count < colors.Length - 1 ? count++ : count = 0;
                    }
                    Array.Reverse(colors);
                }

                dgv.ClearSelection();
            }
        }

        public void DefaultStyle()
        {
            if (Global.DefaultFormColors == 0) DefaultStyle0();
            if (Global.DefaultFormColors == 1) DefaultStyle1();
            if (Global.DefaultFormColors == 2) DefaultStyle2();
        }

        #endregion


        #region Методы стилей по умолчанию

        private void DefaultStyle0()
        {
            ResetFormColor();

            foreach (DataGridView dgv in _dgvs)
            {
                DataGridView dataGridView = new();

                dgv.DefaultCellStyle = null;
                dgv.AlternatingRowsDefaultCellStyle = null;
                dgv.ColumnHeadersDefaultCellStyle = null;
                dgv.RowHeadersDefaultCellStyle = null;
                // Не может быть null
                dgv.BackgroundColor = dataGridView.BackgroundColor;

                foreach (DataGridViewRow row in dgv.Rows)
                {
                    row.DefaultCellStyle = null;
                    foreach (DataGridViewColumn column in dgv.Columns)
                    {
                        column.DefaultCellStyle = null;
                    }

                    foreach(DataGridViewCell cell in row.Cells)
                    {
                        cell.Style = null;
                    }
                }
            }

        }

        private void DefaultStyle1()
        {
            ResetFormColor();

            foreach (DataGridView dgv in _dgvs)
            {
                DataGridView dataGridView = new();
                DataGridViewCellStyle col_style = new();
                DataGridViewCellStyle row_style = new();

                dgv.DefaultCellStyle = dataGridView.DefaultCellStyle;
                dgv.AlternatingRowsDefaultCellStyle = dataGridView.AlternatingRowsDefaultCellStyle;
                dgv.ColumnHeadersDefaultCellStyle = dataGridView.ColumnHeadersDefaultCellStyle;
                dgv.RowHeadersDefaultCellStyle = dataGridView.RowHeadersDefaultCellStyle;
                dgv.BackgroundColor = dataGridView.BackgroundColor;

                foreach (DataGridViewRow row in dgv.Rows)
                {
                    row.DefaultCellStyle = row_style;

                    foreach (DataGridViewColumn column in dgv.Columns)
                    {
                        column.DefaultCellStyle = col_style;
                    }

                    foreach (DataGridViewCell cell in row.Cells)
                    {
                        cell.Style = new();
                    }
                }
            }

        }

        private void DefaultStyle2()
        {
            ResetFormColor();

            for (int i = 0; i < _dgvs.Count; i++)
            {
                foreach (DataGridViewRow row in _dgvs[i].Rows)
                {
                    row.DefaultCellStyle = null;
                    foreach (DataGridViewColumn column in _dgvs[i].Columns)
                    {
                        column.DefaultCellStyle = null;
                    }

                    foreach (DataGridViewCell cell in row.Cells)
                    {
                        cell.Style = null;
                    }
                }


                _dgvs[i].BackgroundColor = _defDvgBackgroundColors[i];

                _dgvs[i].DefaultCellStyle.BackColor = _defStyles[i].BackColor;
                _dgvs[i].DefaultCellStyle.ForeColor = _defStyles[i].ForeColor;

                _dgvs[i].AlternatingRowsDefaultCellStyle.BackColor = _defAltStyles[i].BackColor;
                _dgvs[i].AlternatingRowsDefaultCellStyle.ForeColor = _defAltStyles[i].ForeColor;

                _dgvs[i].ColumnHeadersDefaultCellStyle.BackColor = _defColumnStyles[i].BackColor;
                _dgvs[i].ColumnHeadersDefaultCellStyle.ForeColor = _defColumnStyles[i].ForeColor;

                _dgvs[i].RowHeadersDefaultCellStyle.BackColor = _defRowStyles[i].BackColor;
                _dgvs[i].RowHeadersDefaultCellStyle.ForeColor = _defRowStyles[i].ForeColor;
            }
        }

        private void ResetFormColor()
        {
            MenuStrip strip = new();
            Form form = new();

            _menustrip.BackColor = strip.BackColor;
            _menustrip.ForeColor = strip.ForeColor;

            _form.BackColor = form.BackColor;
            _form.ForeColor = form.ForeColor;

            GroupBox groupBox = new GroupBox();
            foreach (GroupBox grp in _gbxs)
            {
                grp.ForeColor = groupBox.ForeColor;
            }
        }

        private void ResetColors()
        {
            DefaultStyle();
        }

        #endregion


    }
}
